<?php

declare(strict_types=1);

namespace Dotclear\Plugin\HideStoreUpdates;

use ArrayObject;
use Dotclear\App;
use Dotclear\Core\Backend\ModulesList;
use Dotclear\Core\Backend\Notices;
use Dotclear\Database\MetaRecord;
use Dotclear\Helper\Html\Form\Submit;
use Dotclear\Helper\Html\Html;
use Dotclear\Module\ModuleDefine;

/**
 * @brief       HideStoreUpdates backend behaviors class.
 * @ingroup     HideStoreUpdates
 *
 * @author      Jean-Christian Denis
 * @copyright   GPL-2.0 https://www.gnu.org/licenses/gpl-2.0.html
 */
class BackendBehaviors
{
    private static bool $cleared = false;

    /**
     * Add "Hide" button on modules list.
     *
     * @param   ModulesList     $list       The modules list
     * @param   ModuleDefine    $define     The current module
     *
     * @return  string  The button or empty string
     */
    public static function adminModulesListGetActionsV2(ModulesList $list, ModuleDefine $define): string
    {
        return in_array($list->getList(), [
            'plugin-update',
            'theme-update',
        ]) ? (new Submit([My::id() . '[' . Html::escapeHTML($define->getId()) . ' ' . $define->get('version') . ']']))->value(__('Hide'))->render() : '';
    }

    /**
     * Hide selected module on modules list.
     *
     * @param   ModulesList                 $list       Th emodules list
     * @param   array<int, ModuleDefine>    $defines    The modules
     * @param   string                      $type       The modules type (plugins or themes)
     */
    public static function adminModulesListDoActions(ModulesList $list, array $defines, string $type): void
    {
        // Hide an update version
        if (empty($_POST[My::id()])
         || !is_array($_POST[My::id()])) {
            return;
        }

        $type   = str_starts_with($type, 'theme') ? 'themes' : 'plugins';
        $update = key($_POST[My::id()]);
        $tmp    = explode(' ', $update);
        $module = $tmp[0];

        $logs = self::getLogs($type);
        if (!$logs->isEmpty()) {
            $ids = [];
            while ($logs->fetch()) {
                if ($update == $logs->f('log_msg')) {
                    // Same module version allready hidden
                    return;
                }
                if (str_starts_with($logs->f('log_msg'), $module)) {
                    // Same module but older version
                    $ids[] = $logs->f('log_id');
                }
            }
            if (!empty($ids)) {
                // Delete older versions
                App::log()->deLogs($ids);
            }
        }
        if (!empty($update)) {
            // Add module version
            $cur = App::log()->openLogCursor();
            $cur->setField('log_table', My::id() . '_' . $type);
            $cur->setField('log_msg', $update);
            App::log()->addLog($cur);

            Notices::AddSuccessNotice(sprintf(__('Module %s successfully hidden.'), $update));
        }
    }

    /**
     * Hide modules from store update list.
     *
     * @param   string                          $type       The modules type (plugins or themes)
     * @param   ArrayObject<int, ModuleDefine>  $defines    The modules
     */
    public static function afterCheckStoreUpdate(string $type, ArrayObject $defines): void
    {
        if (!self::$cleared && !empty($_GET['nocache'])) {
            // Clear (once) modules versions on manual check
            $logs = self::getLogs($type);
            if (!$logs->isEmpty()) {
                $ids = [];
                while ($logs->fetch()) {
                    $ids[] = (int) $logs->f('log_id');
                }
                App::log()->delLogs($ids);
                self::$cleared = true;

                return;
            }
        }

        $logs = self::getLogs($type);
        if ($logs->isEmpty()) {
            return;
        }

        $hiddens = [];
        while ($logs->fetch()) {
            $hiddens[] = $logs->f('log_msg');
        }

        foreach ($defines as $k => $define) {
            if (in_array($define->getId() . ' ' . $define->get('version'), $hiddens)) {
                $defines->offsetUnset($k);
            }
        }
    }

    private static function getLogs(string $type): MetaRecord
    {
        return App::log()->getLogs([
            'log_table' => My::id() . '_' . $type,
            'blog_id'   => $type == 'plugins' ? '*' : App::blog()->id(),
        ]);
    }
}
